// src/backend/schedulable.hh - Copyright 2005, 2006, University
//               of Padova, dept. of Pure and Applied
//               Mathematics
//
// This file is part of SGPEMv2.
//
// This is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// SGPEMv2 is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with SGPEMv2; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

#ifndef SCHEDULABLE_HH
#define SCHEDULABLE_HH 1

#include "config.h"
#include "glibmm/ustring.h"

namespace sgpem
{
  class Schedulable;

  /** \brief An entity that can use the processor
   *
   * This abstract class describes an entity that can use the processor.  It
   * describes the parameters that don't change during a simulation.
   *
   * For a reference of a schedulable among its current status see referenced
   * class.
   *
   * \see SchedulableStatus
   */
  class SG_DLLEXPORT Schedulable
  {
  public:
    /** \brief Create a new object with the given parameters */
    Schedulable(const Glib::ustring& name, const unsigned int& arrival,
                const unsigned int& total, const int& priority);
    virtual ~Schedulable();

    /** \brief Returns the arrival time for this process
     *
     * The arrival time of a process is the number of time units, starting
     * from instant zero, at which the process is added on the queue.
     */
    virtual unsigned int get_arrival_time() const;

    /** \brief Returns the amount of CPU time this process is going to require */
    unsigned int get_total_cpu_time() const;

    /** \brief Returns the priority of this process
     *
     * The priority of a process is a number assigned to it when it is
     * spawned, and never changes for its lifetime.
     */
    int get_priority() const;

    /** \brief Returns a string representing this object
     *
     * The name of a process is a human readable string assigned to it by the
     * user, that allows it to be quickly recognized.
     */
    Glib::ustring get_name() const;

    /** \brief Returns the type of the process
     *
     * This is an abstract method as the schedulable type is defined by the
     * concrete process, thread, or any other schedulable entity.
     */
    virtual Glib::ustring get_type() const = 0;

  private:
    Glib::ustring _name;
    unsigned int _arrival_time;
    unsigned int _total_time;
    int _priority;
  };
}

#endif
