// src/backend/schedulable_list.hh - Copyright 2005, 2006, University
//               of Padova, dept. of Pure and Applied
//               Mathematics
//
// This file is part of SGPEMv2.
//
// This is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// SGPEMv2 is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with SGPEMv2; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

#ifndef SCHEDULABLE_LIST_HH
#define SCHEDULABLE_LIST_HH 1

#include "config.h"

#include <list>

#include "schedulable_status.hh"
#include "../templates/smartp.hh"


namespace sgpem
{
  class SchedulableList;

  class SG_DLLEXPORT SchedulableList
  {
  public:
    SchedulableList();
    bool operator==(const SchedulableList&) const;
    bool has_same_objects(const SchedulableList& dx) const;

    /** \brief Returns a pointer to the first element
     *
     * This function returns a pointer to the first element, or null if the
     * queue is empty.
     *
     * It is very important not to delete these pointers as their deallocation
     * is managed by the queue.
     */
    SchedulableStatus* top();

    /** \brief Returns a pointer to the last element
     * \see top
     */
    SchedulableStatus* bottom();

    /** \brief Adds an element at the top of the queue */
    void add_at_top(const SchedulableStatus&);

    /** \brief Adds an element at the end of the queue */
    void add_at_bottom(const SchedulableStatus&);

    /** \brief Removes */
/**
	Removes an element from the list. Returns a smart pointer a copy of it or to NULL if
	"position" is out of range.

	Ex.	remove(0);			removes the top of the list
	Ex. remove(size()-1)	removes the bottom of the list
*/

    memory::smart_ptr<sgpem::SchedulableStatus> remove(const unsigned int& position);
    bool insert_at(const unsigned int&, const unsigned int&);
    unsigned int size() const;
    SchedulableStatus* get_item_at(const uint&);
    const SchedulableStatus* get_item_at(const uint&) const;
    void clear();

    /** \brief This method swaps two elements given their list positions
     *
     * At the present moment, this function shouldn't throw any exception, but
     * just do nothing when the parameters don't make sense (e.g. one of the
     * positions is outside range [0,this.size()]).
     *
     * However, it should work the same either if positionA is greater than or
     * less than positionB.
     *
     * In the future, this method could throw an OutOfRange exception.
     *
     * \param positionA The position of the first element to swap
     * \param positionB The position of the second element to swap
     */
     void swap(unsigned int positionA, unsigned int positionB) throw();

  private:
    std::list<SchedulableStatus> _list;
  };
}

#endif //SCHEDULABLE_LIST_HH
