// src/backend/schedulable_status.hh - Copyright 2005, 2006, University
//               of Padova, dept. of Pure and Applied
//               Mathematics
//
// This file is part of SGPEMv2.
//
// This is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// SGPEMv2 is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with SGPEMv2; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

#ifndef SCHEDULABLESTATUS_HH
#define SCHEDULABLESTATUS_HH 1

#include "config.h"
#include "schedulable.hh"

namespace sgpem
{
  class SchedulableStatus;

  /** \brief Desribes the state of a schedulable entity in a particular moment
   * of the simulation
   *
   * This class stores part of the information deeded by the Scheduler to
   * manage processes and other ones.
   *
   * Objects of type SchedulableStatus are created by the Scheduler and are
   * destroyed by SimulationStatus if they are linked to it or by the Scheduler.
   */
  class SG_DLLEXPORT SchedulableStatus
  {
  public:
    /** \brief This flag describes the actual state of the schedulable
     *
     * You can think of this flag as the particular stack in the OS where the
     * process are placed during their lifetime.  In the OS there are three
     * main stacks that are Running processes, Read processes, and Blocked
     * processes. These are emulated in a single list by this flag.
     */
    enum state
    {
      state_running    = 1<<0,
      state_ready      = 1<<1,
      state_blocked    = 1<<2,
      state_future     = 1<<3,
      state_terminated = 1<<4
    };

    /** \brief Object constructor */
    SchedulableStatus(const Schedulable& obj);

    //SchedulableStatus(const SchedulableStatus& obj);	//copy constructor

    /** \brief Verify if two instances represents the same situation
     *
     * This trivial overloading of the equal operator simply verifies if the
     * actual represented process is the same, and if the status is also the
     * same.
     */
    bool operator==(const SchedulableStatus&) const;

    /** \brief Returns the remaining CPU time */
    int get_cpu_time_left() const;

    /** \brief Decrements cpu time left by this amount until it reaches zero */
    void give_cpu_time(const int& time);

    /** \brief Sets the time when this process was last scheduled */
    void set_last_scheduled(const int& time);

    /** \brief Gets the time when this process was last scheduled */
    int get_last_scheduled() const;

    /** \brief Returns the schedule stack in which this process resides
     * \see state */
    state get_state() const;

    /** \brief Sets the state of this process
     * \see state */
    void set_state(state s);

    /** \brief Returns a pointer to the schedulable object
     *
     * This function returns a pointer to the actual schedable object
     * represented, along with its status, by this instance.
     */
    const Schedulable* get_schedulable() const;

  private:
    const Schedulable* _ref;
    int _last;
    int _time_left;
    state _my_state;
  };
}

#endif
